from datetime import datetime

from sqlalchemy import Boolean, Column, DateTime, Float, JSON, String, Text

from app.db.base import Base


class Memory(Base):
    __tablename__ = "memories"

    id = Column(String, primary_key=True, index=True)
    type = Column(String, index=True)
    scope = Column(String, index=True)
    entity_id = Column(String, index=True, nullable=True)
    content = Column(Text, nullable=False)
    metadata_json = Column("metadata", JSON, nullable=True)
    tags = Column(JSON, nullable=True)
    importance = Column(Float, default=0.0)
    created_at = Column(DateTime, default=datetime.utcnow)
    last_accessed = Column(DateTime, nullable=True)
    ttl = Column(DateTime, nullable=True)
    embedding = Column(Text, nullable=True)


class Rule(Base):
    __tablename__ = "rules"

    id = Column(String, primary_key=True, index=True)
    if_json = Column(JSON, nullable=False)
    then_json = Column(JSON, nullable=False)
    guardrails_json = Column(JSON, nullable=True)
    active = Column(Boolean, default=True)
    created_at = Column(DateTime, default=datetime.utcnow)


class Event(Base):
    __tablename__ = "events"

    id = Column(String, primary_key=True, index=True)
    event_type = Column(String, index=True, nullable=False)
    tool_name = Column(String, index=True, nullable=True)
    scope = Column(String, index=True, nullable=True)
    entity_id = Column(String, index=True, nullable=True)
    payload = Column(JSON, nullable=True)
    created_at = Column(DateTime, default=datetime.utcnow, index=True)



from fastapi import FastAPI

from app.api.routes import api_router
from app.db.base import Base
from app.db.session import engine
from app import models  # noqa: F401  # ensure models are imported for metadata

app = FastAPI(title="Memory OS (local V0)")


@app.on_event("startup")
def on_startup():
    Base.metadata.create_all(bind=engine)


app.include_router(api_router)



from app.db.base import Base
from app.db.session import engine
from app import models  # noqa: F401  # ensure models are registered


def init_memory_os_schema() -> None:
    """Create required tables if they do not already exist."""
    Base.metadata.create_all(bind=engine)



from datetime import datetime, timedelta
from typing import Any, Callable, List, Optional

from fastapi import HTTPException
from sqlalchemy.orm import Session

from app.models.memory import Memory, Rule
from app.schemas.memory import (
    ForgetIn,
    MemoryOut,
    PlanIn,
    ReflectIn,
    RememberIn,
    Scope,
)
from app.services.embedding import embed_text, str_to_vec, vec_to_str
from app.services.pii import redact_pii
from app.services.scoring import cosine_sim, lexical_sim
from app.utils.time import new_id, now_utc

EmbedFn = Callable[[str], Any]


def remember_memory(
    db: Session,
    payload: RememberIn,
    *,
    embed_fn: EmbedFn = embed_text,
    redact: bool = True,
) -> MemoryOut:
    text = payload.content
    pii_tags: List[str] = []
    if redact:
        text, pii_tags = redact_pii(payload.content)
    embedding = embed_fn(text)
    memory_id = new_id()

    ttl = None
    if payload.ttl_hours is not None:
        ttl = now_utc() + timedelta(hours=payload.ttl_hours)

    memory = Memory(
        id=memory_id,
        type=payload.type,
        scope=payload.scope,
        entity_id=payload.entity_id,
        content=text,
        metadata_json=payload.metadata or {},
        tags=(payload.tags or []) + pii_tags,
        importance=payload.importance or 0.0,
        created_at=now_utc(),
        ttl=ttl,
        embedding=vec_to_str(embedding),
    )
    db.add(memory)
    db.commit()
    db.refresh(memory)

    return MemoryOut(
        id=memory.id,
        content=memory.content,
        type=memory.type,
        scope=memory.scope,
        entity_id=memory.entity_id,
        tags=memory.tags,
        metadata=memory.metadata_json or {},
    )


def recall_memories(
    db: Session,
    *,
    query: str,
    k: int = 8,
    scope: Optional[Scope] = None,
    entity_id: Optional[str] = None,
    as_of: Optional[str] = None,
    embed_fn: EmbedFn = embed_text,
) -> List[MemoryOut]:
    query_embedding = embed_fn(query)

    q = db.query(Memory)
    if scope:
        q = q.filter(Memory.scope == scope)
    if entity_id:
        q = q.filter(Memory.entity_id == entity_id)

    if as_of:
        try:
            as_of_dt = datetime.fromisoformat(as_of)
            q = q.filter(Memory.created_at <= as_of_dt)
        except ValueError:
            raise HTTPException(status_code=400, detail="Invalid as_of datetime format. Use ISO8601.")

    memories = q.all()
    results = []
    current_time = now_utc()

    for memory in memories:
        if memory.ttl and memory.ttl < current_time:
            continue
        mem_vec = str_to_vec(memory.embedding) if memory.embedding else None
        vector_score = cosine_sim(query_embedding, mem_vec)
        lexical_score = lexical_sim(query, memory.content)
        score = 0.7 * vector_score + 0.3 * lexical_score
        results.append((score, memory))

    results.sort(key=lambda item: item[0], reverse=True)
    top_results = results[:k]

    for _, memory in top_results:
        memory.last_accessed = current_time
    db.commit()

    return [
        MemoryOut(
            id=memory.id,
            content=memory.content,
            type=memory.type,
            scope=memory.scope,
            entity_id=memory.entity_id,
            score=float(score),
            tags=memory.tags,
            metadata=memory.metadata_json or {},
        )
        for score, memory in top_results
    ]


def reflect_session(db: Session, payload: ReflectIn, *, embed_fn: EmbedFn = embed_text):
    query = (
        db.query(Memory)
        .filter(
            Memory.scope == "session",
            Memory.entity_id == payload.session_id,
            Memory.type == "episodic",
        )
        .order_by(Memory.created_at.desc())
        .limit(20)
    )
    events = query.all()
    if not events:
        return {"ok": False, "summary_id": None, "message": "No episodic memories found for this session."}

    summary_lines = [f"- {m.content[:200]}" for m in reversed(events)]
    summary = "Session summary (naive V0):\n" + "\n".join(summary_lines)

    embedding = embed_fn(summary)
    summary_id = new_id()
    summary_memory = Memory(
        id=summary_id,
        type="semantic",
        scope="session",
        entity_id=payload.session_id,
        content=summary,
        metadata_json={"summary_of": [m.id for m in events]},
        tags=["summary"],
        importance=0.6,
        created_at=now_utc(),
        embedding=vec_to_str(embedding),
    )
    db.add(summary_memory)
    db.commit()

    return {"ok": True, "summary_id": summary_id}


def store_plan(db: Session, payload: PlanIn):
    rule_id = new_id()
    rule = Rule(
        id=rule_id,
        if_json=payload.if_.model_dump(),
        then_json=payload.then.model_dump(),
        guardrails_json=payload.guardrails or {},
        active=True,
        created_at=now_utc(),
    )
    db.add(rule)
    db.commit()
    return {"ok": True, "rule_id": rule_id}


def forget_memory(db: Session, payload: ForgetIn):
    if payload.id:
        memory = db.query(Memory).filter(Memory.id == payload.id).first()
        if not memory:
            raise HTTPException(status_code=404, detail="Memory not found")
        db.delete(memory)
        db.commit()
        return {"ok": True, "deleted": payload.id}

    if payload.policy:
        policy = payload.policy
        if policy.startswith("older_than:"):
            try:
                days_str = policy.split(":", 1)[1]
                days = int(days_str.replace("d", ""))
            except ValueError:
                raise HTTPException(status_code=400, detail="Invalid older_than policy. Example: older_than:90d")
            cutoff = now_utc() - timedelta(days=days)
            deleted = db.query(Memory).filter(Memory.created_at < cutoff).delete()
            db.commit()
            return {"ok": True, "deleted_count": deleted, "policy": policy}
        raise HTTPException(status_code=400, detail="Unsupported policy. Supported: older_than:Nd")

    raise HTTPException(status_code=400, detail="Provide either id or policy in the request body")



from __future__ import annotations

from contextlib import contextmanager
from datetime import datetime
from typing import Any, Callable, Dict, Iterable, List, Optional

from sqlalchemy.orm import Session

from app.db.session import SessionLocal
from app.auto.models import (
    ALLOWED_MEMORY_TYPES,
    ALLOWED_SCOPES,
    Extractor,
    Interaction,
    MemoryCandidate,
)
from app.models.memory import Memory
from app.schemas.memory import (
    EventIn,
    EventResponse,
    ForgetIn,
    MemoryOut,
    PlanCondition,
    PlanIn,
    PlanThen,
    ProspectiveAction,
    ReflectIn,
    RememberIn,
)
from app.services.event_service import handle_event
from app.services.memory_service import (
    forget_memory,
    recall_memories,
    reflect_session,
    remember_memory,
    store_plan,
)
from app.services.embedding import embed_text
from app.utils.text import normalize_text


class MemoryClient:
    """Programmatic interface that mirrors the REST API surface."""

    def __init__(
        self,
        *,
        session_factory: Optional[Callable[[], Session]] = None,
        embed_fn: Optional[Callable[[str], Any]] = None,
        redact_pii: bool = True,
    ):
        self._session_factory = session_factory or SessionLocal
        self._embed_fn = embed_fn or embed_text
        self._redact_pii = redact_pii

    @contextmanager
    def _get_session(self, session: Optional[Session] = None):
        if session is not None:
            yield session
            return
        db = self._session_factory()
        try:
            yield db
        finally:
            db.close()

    def remember(
        self,
        *,
        content: str,
        type: str,
        scope: str,
        entity_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
        importance: float = 0.0,
        ttl_hours: Optional[int] = None,
        metadata: Optional[Dict[str, Any]] = None,
        session: Optional[Session] = None,
    ) -> MemoryOut:
        payload = RememberIn(
            content=content,
            type=type,
            scope=scope,
            entity_id=entity_id,
            tags=tags,
            importance=importance,
            ttl_hours=ttl_hours,
            metadata=metadata,
        )
        with self._get_session(session) as db:
            return remember_memory(db, payload, embed_fn=self._embed_fn, redact=self._redact_pii)

    def recall(
        self,
        *,
        query: str,
        k: int = 8,
        scope: Optional[str] = None,
        entity_id: Optional[str] = None,
        as_of: Optional[datetime | str] = None,
        session: Optional[Session] = None,
    ) -> List[MemoryOut]:
        as_of_value: Optional[str]
        if isinstance(as_of, datetime):
            as_of_value = as_of.isoformat()
        else:
            as_of_value = as_of
        with self._get_session(session) as db:
            return recall_memories(
                db,
                query=query,
                k=k,
                scope=scope,
                entity_id=entity_id,
                as_of=as_of_value,
                embed_fn=self._embed_fn,
            )

    def reflect(
        self,
        *,
        session_id: str,
        budget_tokens: int = 1500,
        session: Optional[Session] = None,
    ):
        payload = ReflectIn(session_id=session_id, budget_tokens=budget_tokens)
        with self._get_session(session) as db:
            return reflect_session(db, payload, embed_fn=self._embed_fn)

    def create_rule(
        self,
        *,
        condition: Dict[str, Any],
        actions: Dict[str, Any],
        guardrails: Optional[Dict[str, Any]] = None,
        session: Optional[Session] = None,
    ):
        plan_condition = PlanCondition(**condition)
        # Allow callers to pass raw action dicts or already-validated models
        if "actions" in actions and isinstance(actions["actions"], Iterable):
            action_models = [ProspectiveAction(**action) for action in actions["actions"]]  # type: ignore[arg-type]
        else:
            raise ValueError("actions dictionary must include an 'actions' list.")
        plan_then = PlanThen(actions=action_models)
        plan_payload = PlanIn(if_=plan_condition, then=plan_then, guardrails=guardrails)
        with self._get_session(session) as db:
            return store_plan(db, plan_payload)

    def record_event(
        self,
        *,
        event_type: str,
        tool_name: Optional[str] = None,
        scope: Optional[str] = None,
        entity_id: Optional[str] = None,
        payload: Optional[Dict[str, Any]] = None,
        session: Optional[Session] = None,
    ) -> EventResponse:
        event_payload = EventIn(
            event_type=event_type,
            tool_name=tool_name,
            scope=scope,
            entity_id=entity_id,
            payload=payload,
        )
        with self._get_session(session) as db:
            return handle_event(db, event_payload)

    def forget(
        self,
        *,
        id: Optional[str] = None,
        policy: Optional[str] = None,
        reason: Optional[str] = "unspecified",
        session: Optional[Session] = None,
    ):
        payload = ForgetIn(id=id, policy=policy, reason=reason)
        with self._get_session(session) as db:
            return forget_memory(db, payload)

    def auto_ingest(
        self,
        *,
        interactions: List[Interaction],
        extractor: Extractor,
        scope: Optional[str] = None,
        entity_id: Optional[str] = None,
        max_memories: Optional[int] = None,
        min_importance: Optional[float] = None,
        session: Optional[Session] = None,
    ) -> List[MemoryOut]:
        if not interactions:
            return []
        if extractor is None:
            raise ValueError("auto_ingest requires a non-null extractor")

        candidates = extractor(interactions)
        normalized: List[MemoryCandidate] = []
        for candidate in candidates:
            content = (candidate.content or "").strip()
            if not content:
                continue

            ctype = candidate.type if candidate.type in ALLOWED_MEMORY_TYPES else "semantic"
            importance = candidate.importance
            if min_importance is not None and importance is not None and importance < min_importance:
                continue

            scoped = candidate.model_copy()
            scoped.content = content
            scoped.type = ctype

            # Clamp scope to allowed values; default to provided scope or "session".
            effective_scope = scope or "session"
            if scoped.scope in ALLOWED_SCOPES:
                effective_scope = scoped.scope
            scoped.scope = effective_scope

            # Use candidate entity_id when provided, else fallback to outer entity_id.
            scoped.entity_id = scoped.entity_id or entity_id

            normalized.append(scoped)

        # Deduplicate against existing memories per (scope, entity_id) and within this batch
        with self._get_session(session) as db:
            seen_by_key: Dict[tuple, set] = {}

            def get_seen(scope_val: str, entity_val: Optional[str]) -> set:
                key = (scope_val, entity_val)
                if key not in seen_by_key:
                    q = db.query(Memory).filter(Memory.scope == scope_val)
                    if entity_val:
                        q = q.filter(Memory.entity_id == entity_val)
                    existing = q.all()
                    seen_by_key[key] = {normalize_text(mem.content) for mem in existing}
                return seen_by_key[key]

            deduped: List[MemoryCandidate] = []
            for candidate in normalized:
                eff_scope = candidate.scope or "session"
                eff_entity = candidate.entity_id
                seen = get_seen(eff_scope, eff_entity)
                norm_content = normalize_text(candidate.content)
                if norm_content in seen:
                    continue
                seen.add(norm_content)
                deduped.append(candidate)

            # Apply max_memories after dedupe, sorted by importance desc (None -> 0.0)
            deduped.sort(key=lambda c: c.importance if c.importance is not None else 0.0, reverse=True)
            if max_memories is not None:
                deduped = deduped[:max_memories]

            stored: List[MemoryOut] = []
            for candidate in deduped:
                stored.append(
                    self.remember(
                        content=candidate.content,
                        type=candidate.type,
                        scope=candidate.scope or "session",
                        entity_id=candidate.entity_id,
                        tags=candidate.tags,
                        importance=candidate.importance or 0.0,
                        metadata=candidate.metadata,
                        session=db,
                    )
                )
        return stored



from typing import List, Optional

from fastapi import APIRouter, Depends
from sqlalchemy.orm import Session

from app.api.deps.db import get_db
from app.core import MemoryClient
from app.schemas.memory import (
    EventIn,
    EventResponse,
    ForgetIn,
    MemoryOut,
    PlanIn,
    ReflectIn,
    RememberIn,
    Scope,
)

router = APIRouter(prefix="/v0")


@router.post("/remember", response_model=MemoryOut)
def remember(payload: RememberIn, db: Session = Depends(get_db)):
    client = MemoryClient()
    return client.remember(
        content=payload.content,
        type=payload.type,
        scope=payload.scope,
        entity_id=payload.entity_id,
        tags=payload.tags,
        importance=payload.importance or 0.0,
        ttl_hours=payload.ttl_hours,
        metadata=payload.metadata,
        session=db,
    )


@router.get("/recall", response_model=List[MemoryOut])
def recall(
    query: str,
    k: int = 8,
    scope: Optional[Scope] = None,
    entity_id: Optional[str] = None,
    as_of: Optional[str] = None,
    db: Session = Depends(get_db),
):
    client = MemoryClient()
    return client.recall(
        query=query,
        k=k,
        scope=scope,
        entity_id=entity_id,
        as_of=as_of,
        session=db,
    )


@router.post("/reflect")
def reflect(payload: ReflectIn, db: Session = Depends(get_db)):
    client = MemoryClient()
    return client.reflect(
        session_id=payload.session_id,
        budget_tokens=payload.budget_tokens,
        session=db,
    )


@router.post("/plan")
def plan(payload: PlanIn, db: Session = Depends(get_db)):
    client = MemoryClient()
    return client.create_rule(
        condition=payload.if_.model_dump(),
        actions=payload.then.model_dump(),
        guardrails=payload.guardrails,
        session=db,
    )


@router.post("/forget")
def forget(payload: ForgetIn, db: Session = Depends(get_db)):
    client = MemoryClient()
    return client.forget(
        id=payload.id,
        policy=payload.policy,
        reason=payload.reason,
        session=db,
    )


@router.post("/event", response_model=EventResponse)
def ingest_event(payload: EventIn, db: Session = Depends(get_db)):
    client = MemoryClient()
    return client.record_event(
        event_type=payload.event_type,
        tool_name=payload.tool_name,
        scope=payload.scope,
        entity_id=payload.entity_id,
        payload=payload.payload,
        session=db,
    )



"""
Backward compatibility alias to support legacy imports (`import memory_os`).
Prefer importing from `mengram`.
"""

from mengram import MemoryClient, init_memory_os_schema  # type: ignore

__all__ = ["MemoryClient", "init_memory_os_schema"]



"""
Primary public package surface for mengram.

Exports:
- MemoryClient: programmatic API for memories/rules/events
- init_memory_os_schema: creates required tables (idempotent)
- Interaction / MemoryCandidate: data models for auto-ingest
- LLMMemoryExtractor: reference LLM-based extractor
"""

from app.core import MemoryClient
from app.db.init_db import init_memory_os_schema
from app.auto import Interaction, MemoryCandidate, LLMMemoryExtractor, interactions_from_dicts

__all__ = [
    "MemoryClient",
    "init_memory_os_schema",
    "Interaction",
    "MemoryCandidate",
    "LLMMemoryExtractor",
    "interactions_from_dicts",
]



from typing import Optional

import numpy as np

_embedder = None


def get_embedder():
    """Lazy-load the sentence-transformers model."""
    global _embedder
    if _embedder is None:
        try:
            from sentence_transformers import SentenceTransformer
        except ImportError as exc:
            raise RuntimeError(
                "sentence-transformers is required. Install it with 'pip install sentence-transformers'."
            ) from exc
        _embedder = SentenceTransformer("all-MiniLM-L6-v2")
    return _embedder


def embed_text(text: str) -> np.ndarray:
    model = get_embedder()
    return model.encode([text], normalize_embeddings=True)[0].astype(np.float32)


def vec_to_str(vec) -> str:
    """
    Convert an embedding vector to a space-separated string.

    Accepts:
    - numpy arrays
    - Python lists / tuples
    - any iterable of numeric values
    """
    # If the object already supports .tolist(), use it (numpy arrays etc.)
    if hasattr(vec, "tolist"):
        values = vec.tolist()
    else:
        # Fallback: try to interpret it as an iterable
        values = list(vec)

    return " ".join(str(x) for x in values)


def str_to_vec(data: Optional[str]) -> np.ndarray:
    if not data:
        return np.zeros(384, dtype=np.float32)
    floats = [float(x) for x in data.split()]
    return np.array(floats, dtype=np.float32)



import re


_whitespace_re = re.compile(r"\s+")


def normalize_text(text: str) -> str:
    """
    Lightweight normalization for deduping:
    - trim
    - collapse whitespace
    - lowercase
    """
    collapsed = _whitespace_re.sub(" ", text.strip())
    return collapsed.lower()



import json
import unittest

from mengram import Interaction, LLMMemoryExtractor, MemoryCandidate
from app.auto.extractors import MemoryExtractionError


def fake_llm_ok(prompt: str, model=None, temperature=None) -> str:
    # Return two candidates as JSON
    return json.dumps(
        [
            {
                "content": "User prefers morning deliveries.",
                "type": "semantic",
                "importance": 0.8,
                "tags": ["preference"],
            },
            {
                "content": "System outage occurred at noon.",
                "type": "episodic",
                "importance": 0.3,
                "tags": ["incident"],
            },
        ]
    )


def fake_llm_bad_json(prompt: str, model=None, temperature=None) -> str:
    return "not-json"


def fake_llm_object(prompt: str, model=None, temperature=None) -> str:
    return json.dumps({"memories": []})


def fake_llm_overflow(prompt: str, model=None, temperature=None) -> str:
    return json.dumps(
        [
            {"content": f"Memory {i}", "type": "semantic", "importance": 1.0}
            for i in range(5)
        ]
    )


class LLMMemoryExtractorTest(unittest.TestCase):
    def setUp(self):
        self.interactions = [
            Interaction(role="user", content="Hi"),
            Interaction(role="assistant", content="Hello"),
        ]

    def test_happy_path_parses_candidates(self):
        extractor = LLMMemoryExtractor(llm_client=fake_llm_ok, max_memories=5)
        candidates = extractor(self.interactions)
        self.assertEqual(len(candidates), 2)
        self.assertIsInstance(candidates[0], MemoryCandidate)
        self.assertEqual(candidates[0].content, "User prefers morning deliveries.")
        self.assertEqual(candidates[0].type, "semantic")

    def test_invalid_json_raises(self):
        extractor = LLMMemoryExtractor(llm_client=fake_llm_bad_json)
        with self.assertRaises(MemoryExtractionError):
            extractor(self.interactions)

    def test_non_array_raises(self):
        extractor = LLMMemoryExtractor(llm_client=fake_llm_object)
        with self.assertRaises(MemoryExtractionError):
            extractor(self.interactions)

    def test_max_memories_cap(self):
        extractor = LLMMemoryExtractor(llm_client=fake_llm_overflow, max_memories=2)
        candidates = extractor(self.interactions)
        self.assertEqual(len(candidates), 2)


if __name__ == "__main__":
    unittest.main()



from typing import Any, Dict, List, Literal, Optional

from pydantic import BaseModel, Field, ConfigDict

MemType = Literal["episodic", "semantic", "profile", "tool"]
Scope = Literal["session", "user", "org"]


class RememberIn(BaseModel):
    content: str
    type: MemType
    scope: Scope
    entity_id: Optional[str] = None
    tags: Optional[List[str]] = None
    importance: Optional[float] = 0.0
    ttl_hours: Optional[int] = None
    metadata: Optional[dict] = None


class MemoryOut(BaseModel):
    id: str
    content: str
    type: MemType
    scope: Scope
    entity_id: Optional[str]
    score: Optional[float] = None
    tags: Optional[List[str]] = None
    metadata: Optional[dict] = None


class ReflectIn(BaseModel):
    session_id: str
    budget_tokens: int = 1500


class PlanCondition(BaseModel):
    event_type: str
    tool_name: Optional[str] = None
    scope: Optional[Scope] = None
    entity_id: Optional[str] = None
    window_minutes: int = Field(..., gt=0)
    threshold_count: int = Field(..., gt=0)


class ProspectiveAction(BaseModel):
    type: str
    channel: Optional[str] = None
    target: Optional[str] = None
    message: Optional[str] = None
    content: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None


class PlanThen(BaseModel):
    actions: List[ProspectiveAction]


class PlanIn(BaseModel):
    model_config = ConfigDict(populate_by_name=True)

    if_: PlanCondition = Field(..., alias="if")
    then: PlanThen
    guardrails: Optional[dict] = None


class ForgetIn(BaseModel):
    id: Optional[str] = None
    policy: Optional[str] = None
    reason: Optional[str] = "unspecified"


class EventIn(BaseModel):
    event_type: str
    tool_name: Optional[str] = None
    scope: Optional[Scope] = None
    entity_id: Optional[str] = None
    payload: Optional[Dict[str, Any]] = None


class TriggeredAction(BaseModel):
    type: str
    channel: Optional[str] = None
    target: Optional[str] = None
    message: Optional[str] = None
    content: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None


class TriggeredRuleOut(BaseModel):
    rule_id: str
    actions: List[TriggeredAction]


class EventResponse(BaseModel):
    ok: bool = True
    triggered: List[TriggeredRuleOut]
